@@ -1,7 +1,7 @@
 //! Error handling for application manifest embedding.
 
 use std::fmt::{self, Display, Formatter};
-use std::io;
+use std::io::{self, ErrorKind};
 
 /// The error type which is returned when application manifest embedding fails.
 #[derive(Debug)]
@@ -51,7 +51,7 @@ impl From<Error> for io::Error {
     fn from(e: Error) -> Self {
         match e.repr {
             Repr::IoError(ioe) => ioe,
-            _ => io::Error::other(e),
+            _ => io::Error::new(ErrorKind::Other, e),
         }
     }
 }
