@@ -101,13 +101,13 @@ impl QuadBez {
 
     /// Is this quadratic Bezier curve finite?
     #[inline]
-    pub fn is_finite(&self) -> bool {
+    pub const fn is_finite(&self) -> bool {
         self.p0.is_finite() && self.p1.is_finite() && self.p2.is_finite()
     }
 
     /// Is this quadratic Bezier curve NaN?
     #[inline]
-    pub fn is_nan(&self) -> bool {
+    pub const fn is_nan(&self) -> bool {
         self.p0.is_nan() || self.p1.is_nan() || self.p2.is_nan()
     }
 }
@@ -163,7 +163,7 @@ pub(crate) struct FlattenParams {
     a2: f64,
     u0: f64,
     uscale: f64,
-    /// The number of subdivisions * 2 * sqrt_tol.
+    /// The number of `subdivisions * 2 * sqrt_tol`.
     pub(crate) val: f64,
 }
 
@@ -273,7 +273,12 @@ impl ParamCurveArclen for QuadBez {
 
         let v0 = 0.25 * a2 * a2 * b * (2.0 * sabc - c2) + sabc;
         // TODO: justify and fine-tune this exact constant.
-        if ba_c2 < 1e-13 {
+        // The factor of a2 here is a little arbitrary: we really want
+        // to test whether ba_c2 is small, but it's also important for
+        // this comparison to be scale-invariant. We chose a2 (instead of,
+        // for example, c2 on the rhs) because it's unchanged under
+        // reversing the parametrization.
+        if ba_c2 * a2 < 1e-13 {
             // This case happens for Béziers with a sharp kink.
             v0
         } else {
@@ -328,6 +333,9 @@ impl ParamCurveNearest for QuadBez {
         let mut r_best = None;
         let mut t_best = 0.0;
         let mut need_ends = false;
+        if roots.is_empty() {
+            need_ends = true;
+        }
         for &t in &roots {
             need_ends |= try_t(self, p, &mut t_best, &mut r_best, t);
         }
@@ -508,6 +516,19 @@ mod tests {
         verify(q.nearest((0.0, 1.0).into(), 1e-3), 0.5);
     }
 
+    #[test]
+    fn quadbez_nearest_rounding_panic() {
+        let quad = QuadBez::new(
+            (-1.0394736842105263, 0.0),
+            (0.8210526315789474, -1.511111111111111),
+            (0.0, 1.9333333333333333),
+        );
+        let test = Point::new(-1.7976931348623157e308, 0.8571428571428571);
+        // accuracy ignored
+        let _res = quad.nearest(test, 1e-6);
+        // if we got here then we didn't panic
+    }
+
     #[test]
     fn quadbez_extrema() {
         // y = x^2
@@ -529,4 +550,14 @@ mod tests {
         assert!((extrema[0] - 1.0 / 3.0).abs() < 1e-6);
         assert!((extrema[1] - 2.0 / 3.0).abs() < 1e-6);
     }
+
+    // A regression test for #477: the approximate-linearity test for
+    // using the analytic solution needs to be scale-invariant.
+    #[test]
+    fn perimeter_not_nan() {
+        let q = QuadBez::new((2685., -1251.), (2253., -1303.), (2253., -1303.));
+
+        let len = q.arclen(crate::DEFAULT_ACCURACY);
+        assert!(len.is_finite());
+    }
 }
